import 'package:flutter/rendering.dart';

class SliverGridDelegateWithFixedCrossAxisCountAndLoading
    extends SliverGridDelegateWithFixedCrossAxisCount {
  final int itemCount;

  const SliverGridDelegateWithFixedCrossAxisCountAndLoading({
    required super.crossAxisCount,
    required this.itemCount,
    super.mainAxisSpacing = 0.0,
    super.crossAxisSpacing = 0.0,
    super.childAspectRatio = 1.0,
  });

  @override
  SliverGridLayout getLayout(SliverConstraints constraints) {
    final double usableCrossAxisExtent =
        constraints.crossAxisExtent - crossAxisSpacing * (crossAxisCount - 1);
    final double childCrossAxisExtent = usableCrossAxisExtent / crossAxisCount;
    final double childMainAxisExtent = childCrossAxisExtent / childAspectRatio;
    return SliverGridRegularTileLayoutAndLoading(
        crossAxisCount: crossAxisCount,
        mainAxisStride: childMainAxisExtent + mainAxisSpacing,
        crossAxisStride: childCrossAxisExtent + crossAxisSpacing,
        childMainAxisExtent: childMainAxisExtent,
        childCrossAxisExtent: childCrossAxisExtent,
        reverseCrossAxis:
            axisDirectionIsReversed(constraints.crossAxisDirection),
        fullCrossAccessExtent: usableCrossAxisExtent,
        itemCount: itemCount);
  }
}

class SliverGridDelegateWithMaxCrossAxisExtentAndLoading
    extends SliverGridDelegateWithMaxCrossAxisExtent {
  final int itemCount;

  const SliverGridDelegateWithMaxCrossAxisExtentAndLoading({
    required super.maxCrossAxisExtent,
    required this.itemCount,
    super.mainAxisSpacing = 0.0,
    super.crossAxisSpacing = 0.0,
    super.childAspectRatio = 1.0,
  });

  @override
  SliverGridLayout getLayout(SliverConstraints constraints) {
    final int crossAxisCount =
        (constraints.crossAxisExtent / (maxCrossAxisExtent + crossAxisSpacing))
            .ceil();
    final double usableCrossAxisExtent =
        constraints.crossAxisExtent - crossAxisSpacing * (crossAxisCount - 1);
    final double childCrossAxisExtent = usableCrossAxisExtent / crossAxisCount;
    final double childMainAxisExtent = childCrossAxisExtent / childAspectRatio;
    return SliverGridRegularTileLayoutAndLoading(
      itemCount: itemCount,
      fullCrossAccessExtent: usableCrossAxisExtent,
      crossAxisCount: crossAxisCount,
      mainAxisStride: childMainAxisExtent + mainAxisSpacing,
      crossAxisStride: childCrossAxisExtent + crossAxisSpacing,
      childMainAxisExtent: childMainAxisExtent,
      childCrossAxisExtent: childCrossAxisExtent,
      reverseCrossAxis: axisDirectionIsReversed(constraints.crossAxisDirection),
    );
  }
}

class SliverGridRegularTileLayoutAndLoading
    extends SliverGridRegularTileLayout {
  final int itemCount;
  final double fullCrossAccessExtent;

  const SliverGridRegularTileLayoutAndLoading(
      {required super.crossAxisCount,
      required super.mainAxisStride,
      required super.crossAxisStride,
      required super.childMainAxisExtent,
      required super.childCrossAxisExtent,
      required super.reverseCrossAxis,
      required this.fullCrossAccessExtent,
      required this.itemCount});

  @override
  SliverGridGeometry getGeometryForChildIndex(int index) {
    if (index == itemCount - 1) {
      return SliverGridGeometry(
          scrollOffset: (index ~/ crossAxisCount) * mainAxisStride,
          crossAxisOffset: 0.0,
          mainAxisExtent: childMainAxisExtent,
          crossAxisExtent: fullCrossAccessExtent);
    }

    return super.getGeometryForChildIndex(index);
  }
}
